<?php

if ( ! defined( 'ABSPATH' ) ) { exit; }

//*************** Table ***************
if( ! class_exists( 'WP_List_Table' ) ) {
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

class Image_Annotation_List_Table extends WP_List_Table {
    function __construct(){
		global $status, $page;
			parent::__construct( array(
				'singular'  => esc_html__( 'note', 'demon-image-annotation' ),     //singular name of the listed records
				'plural'    => esc_html__( 'notes', 'demon-image-annotation' ),   //plural name of the listed records
				'ajax'      => false        //does this table support ajax?
	
		) );
    }

  function no_items() {
	esc_html__( 'No notes found.', 'demon-image-annotation' );
  }

  function column_default( $item, $column_name ) {
    switch( $column_name ) { 
        case 'note_img_ID':
			return $item->$column_name;
		case 'note_text':
            return '<strong>'.esc_html__( 'Submitted on :', 'demon-image-annotation' ).'</strong>'.esc_textarea(date( 'y/m/d g:i A', strtotime($item->note_date))).'<br/>'.esc_textarea(nl2br($item->$column_name));
		case 'note_author':
			return '<strong>'.get_avatar($row->note_email, 30, '').esc_textarea($item->$column_name).'</strong><br><a href="mailto:'. esc_textarea($item->note_email).'">'. esc_textarea($item->note_email).'</a>';
		case 'note_response':
			if( (get_option('demon_image_annotation_comments') == '0') ) {
				global $wpdb;
				$list = $wpdb->get_results( $wpdb->prepare("SELECT * from ".DIA_TABLE_COMMENTS." WHERE comment_ID = %s AND comment_content = %s", $item->note_comment_ID, $item->note_text));
				$count;
				foreach ($list as $t) {
					$comment_approved = $t->comment_approved;
					$post = get_post($t->comment_post_ID);
					$posttitle = $post->post_title;
					return esc_textarea($posttitle);
					$count ++;
				}
			}
			if($count == 0) {
				$post = get_post($item->note_post_ID);
				$posttitle = $post->post_title;
				if($posttitle==''){
					return esc_html__( 'Not sync with wordpress comment', 'demon-image-annotation' );
				}else{
					return esc_textarea($posttitle);	
				}
			}
        default:
            return print_r( esc_textarea($item), true ) ; //Show the whole array for troubleshooting purposes
		}
	}

	function get_sortable_columns() {
	  $sortable_columns = array(
		'note_img_ID'  => array('note_img_ID',false),
		'note_author' => array('note_author',false),
		'note_text'   => array('note_text',false)
	  );
	  return $sortable_columns;
	}

	function get_columns(){
			$columns = array(
				'cb'        => '<input type="checkbox" />',
				'note_img_ID' => esc_html__( 'IMG ID', 'imageannotatetable' ),
				'note_author' => esc_html__( 'Author', 'imageannotatetable' ),
				'note_text'    => esc_html__( 'Text', 'imageannotatetable' ),
				'note_response' => esc_html__( 'In Response to', 'imageannotatetable' ),
				'note_action' => esc_html__( 'Action', 'imageannotatetable' )
			);
			 return $columns;
		}
	
	function usort_reorder( $a, $b ) {
	  // If no sort, default to title
	  $orderby = ( ! empty( $_GET['orderby'] ) ) ? $_GET['orderby'] : 'note_ID';
	  // If no order, default to asc
	  $order = ( ! empty($_GET['order'] ) ) ? $_GET['order'] : 'asc';
	  // Determine sort order
	  $result = strcmp( $a[$orderby], $b[$orderby] );
	  // Send final sort direction to usort
	  return ( $order === 'asc' ) ? $result : -$result;
	}
	
	function column_note_action($item){
		$page = isset($_REQUEST['page']) ? sanitize_text_field($_REQUEST['page']) : '';
		$paged = isset($_REQUEST['paged']) ? sanitize_text_field($_REQUEST['paged']) : '';

		$new = '';

		if( (get_option('demon_image_annotation_comments') == '0') ) {
			//sync with wordpress comments
			if($item->comment_approved == '1') {
				$condition = 'Unapprove';
				$text = esc_html__( 'Approve', 'demon-image-annotation' );
			} else if($item->comment_approved == '0') {
				$condition = 'Approve';
				$text = esc_html__( 'Unapprove', 'demon-image-annotation' );
			} else if($item->comment_approved == 'trash') {
				$condition = 'Restore';
				$text = esc_html__( 'Trash', 'demon-image-annotation' );
			}
			
			//not sycn or sync but with old notes
			if($item->comment_approved == ''){
				$condition = 'Notsync';
				if($item->note_comment_ID == 0){
					//deleted comments
					$text = esc_html__( 'Not sync with wordpress comment', 'demon-image-annotation' );
				}else{
					//not Sycn
					$text = '<span style="color:#a00">'.esc_html__( 'Deleted Comment', 'demon-image-annotation' ).'</span>';
				}
			}

			if($item->comment_approved == 'spam'){
				$condition = 'Spam';
				$text = '<span style="color:#a00">'.esc_html__( 'Spam', 'demon-image-annotation' ).'</span>';
			}
		}else{
			if($item->note_approved == '1') {
				$condition = 'Unapprove';
				$text = esc_html__( 'Approve', 'demon-image-annotation' );
			} else {
				$condition = 'Approve';
				$text = esc_html__( 'Unapprove', 'demon-image-annotation' );
			}
		}
		
		if ( current_user_can('moderate_comments') ) {
			$moderate = true;
		}

	  	if($moderate){
			if($condition == 'Notsync' || $condition == 'Spam'){
				$actions = array(
						'delete'    => sprintf('<a href="?page=%s&action=%s&note=%s&paged=%s">'.esc_html__( 'Delete', 'demon-image-annotation' ).'</a>',esc_textarea($page),'delete', esc_attr($item->note_ID), esc_textarea($paged)),
				);
			}else{
				$actions = array(
					esc_textarea($condition)  => sprintf('<a href="?page=%s&action=%s&note=%s&paged=%s">'.esc_textarea($condition).'</a>',esc_textarea($page),esc_textarea(strtolower($condition)),esc_attr($item->note_ID), esc_textarea($paged)),
					'edit'      => sprintf('<a href="?page=%s&action=%s&note=%s&paged=%s">'.esc_html__( 'Edit', 'demon-image-annotation' ).'</a>',esc_textarea($page),'edit',esc_attr($item->note_ID), esc_textarea($paged)),
					'delete'    => sprintf('<a href="?page=%s&action=%s&note=%s&paged=%s">'.esc_html__( 'Delete', 'demon-image-annotation' ).'</a>',esc_textarea($page),'delete', esc_attr($item->note_ID), esc_textarea($paged)),
				);
			}

			return sprintf('%1$s %2$s', $text, $this->row_actions($actions) );
		}else{
			return sprintf('%1$s %2$s', $text, $this->row_actions($actions) );  
		}
	}
	
	function single_row( $a_comment ) {
			global $comment;
			$comment = $a_comment;
			$condition;
			if( (get_option('demon_image_annotation_comments') == '0') ) {
				//sync with wordpress comments
				if($comment->comment_approved == '1') {
					$condition = 'approved';
				} else if($comment->comment_approved == '0') {
					$condition = 'unapproved';
				} else if($comment->comment_approved == 'trash') {
					$condition = 'deleted';
				}
				
				//not sycn or sync but with old notes
				if($comment->comment_approved == ''){
					$condition = 'deleted';
				}
			}else{
				if($comment->note_approved == '1') {
					$condition = 'approved';
				} else {
					$condition = 'unapproved';
				}
			}
			$the_comment_class = join( ' ', get_comment_class( $condition ) );
			
			echo "<tr id='comment-$comment->note_ID' class='$the_comment_class'>";
			echo $this->single_row_columns( $comment );
			echo "</tr>\n";
	}
	
	function get_bulk_actions() {
		$actions = array(
			'delete' => esc_html__( 'Delete' , 'demon-image-annotation'),
			'approve' => esc_html__( 'Approve' , 'demon-image-annotation'),
			'unapprove' => esc_html__( 'Unapprove' , 'demon-image-annotation')
		);
	
		return $actions;
	}
	
	// Handle bulk actions
	function process_bulk_action() {
		global $wpdb;
		
		// Define our data source
		if ( 'delete' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Deleted.' , 'demon-image-annotation').'</strong></p></div>';
		} else if ( 'approve' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Approved.' , 'demon-image-annotation').'</strong></p></div>';
		} else if ( 'restore' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Restored.' , 'demon-image-annotation').'</strong></p></div>';
		} else if ( 'unapprove' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Unapproved.' , 'demon-image-annotation').'</strong></p></div>';
		} else if ( 'edit' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Edit.' , 'demon-image-annotation').'</strong></p></div>';
		} else if ( 'update' === $this->current_action() ) {
			echo '<div class="updated"><p><strong>'.esc_html__( 'Images Note Updated.' , 'demon-image-annotation').'</strong></p></div>';
		}
		
		if (isset($_POST['update_single_note'])) {
			if (!wp_verify_nonce($_POST['_wpnonce'], 'imagenotesactionupdate')) die(esc_html__( 'Update security violated.' , 'demon-image-annotation'));	
			if(sanitize_text_field($_POST['update_single_note']) == "yes") {
				$imgid = sanitize_text_field($_POST['note_ID']);
				$commentid = sanitize_text_field($_POST['note_comment_ID']);
				
				if( (get_option('demon_image_annotation_comments') == '0') ) {
					$wpdb->update(
						DIA_TABLE_COMMENTS,
						array(
							'comment_content' => sanitize_text_field($_POST['note_text']),
							'comment_author' => sanitize_text_field($_POST['note_author']),
							'comment_author_email' => sanitize_text_field($_POST['note_email'])
						),
						array(
							'comment_ID' => $commentid
						)
					);
				}

				$wpdb->update(
					DIA_TABLE_NOTES,
					array(
						'note_author' => sanitize_text_field($_POST['note_author']),
						'note_email' => sanitize_text_field($_POST['note_email']),
						'note_top' => sanitize_text_field($_POST['note_top']),
						'note_left' => sanitize_text_field($_POST['note_left']),
						'note_width' => sanitize_text_field($_POST['note_width']),
						'note_height' => sanitize_text_field($_POST['note_height']),
						'note_text' => sanitize_text_field($_POST['note_text'])
					),
					array(
						'note_ID' => $imgid
					)
				);
			}
		}
	}
	
	function column_cb($item) {
		if ( current_user_can('moderate_comments') ) {
			$moderate = true;
		}
		if($moderate){
			return sprintf(
				'<input type="checkbox" name="note[]" value="%s" />', esc_attr($item->note_ID)
			);
		}
	}
	
	function get_views() {
		global $allnum, $pendingnum, $approvednum;

		$page = isset($_REQUEST['page']) ? sanitize_text_field($_REQUEST['page']) : '';
		$current = ( !empty($_REQUEST['filter']) ? sanitize_text_field($_REQUEST['filter']) : 'all');
		
		$allclass = ($current == 'all' || $current == '' ? ' class="current"' :'');
		$pendingclass = ($current == 'pending' ? ' class="current"' :'');
		$approvedclass = ($current == 'approved' ? ' class="current"' :'');
			
		$status_links = array(
			"all"       => __("<a href='".sprintf('?page=%s',esc_textarea($page))."' ".$allclass.">".esc_html__( 'All' , 'demon-image-annotation')."</a> ".sprintf('(%s)', esc_textarea($allnum))."",'demon-image-annotation'),
			"pending" => __("<a href='".sprintf('?page=%s&filter=%s',esc_textarea($page),'pending')."' ".$pendingclass.">".esc_html__( 'Pending' , 'demon-image-annotation')."</a> ".sprintf('(%s)', esc_textarea($pendingnum))."",'demon-image-annotation'),
			"approved"   => __("<a href='".sprintf('?page=%s&filter=%s',esc_textarea($page),'approved')."' ".$approvedclass.">".esc_html__( 'Approved' , 'demon-image-annotation')."</a> ".sprintf('(%s)', esc_textarea($approvednum))."",'demon-image-annotation')
		);
		return $status_links;
	}
	
	function editNote() {
		if ( 'edit' === $this->current_action() ) {
			$page = isset($_REQUEST['page']) ? sanitize_text_field($_REQUEST['page']) : '';
			$paged = isset($_REQUEST['paged']) ? sanitize_text_field($_REQUEST['paged']) : '';
			$noteid = ( is_array( $_REQUEST['note'] ) ) ? $_REQUEST['note'] : array( $_REQUEST['note'] );
			global $wpdb;
			
			foreach ( $noteid as $id ) {
				$id = absint( sanitize_text_field($id) );
				$result = $wpdb->get_results( $wpdb->prepare("SELECT * from ".DIA_TABLE_NOTES." WHERE note_ID = %s", $id));
				
				?>
                <div class="wrap">
				<form name="dia_update_form" method="post" action="<?php echo sprintf('?page=%s&action=%s&paged=%s',esc_textarea($page),'update',esc_textarea($paged)); ?>">
          
				<input type="hidden" name="update_single_note" value="yes">
				<?php wp_nonce_field('imagenotesactionupdate') ?>
                
				<?php
				//print_r($result);
				foreach ($result as $r) {
					echo '<table class="widefat" width="100%">';
					echo '<thead><tr>';
					echo '<th colspan="2">'.esc_html__( 'Edit Image Note : ' , 'demon-image-annotation').esc_textarea($r->note_img_ID).'<input type="hidden" name="note_ID" value="'.esc_textarea($r->note_ID).'" /><input type="hidden" name="note_comment_ID" value="'.esc_textarea($r->note_comment_ID).'" /></th>';
					echo '</tr></thead>';
					echo '<tbody>';
					echo '<tr>';
					echo '<td width="120">'.esc_html__( 'Image Preview' , 'demon-image-annotation').'</td>';
					echo '<td>';
					
					$postid = $r->note_post_ID;
					
					if($postid == 0){
						//grab post ID from comment
						global $wpdb;
						//$list = $wpdb->get_results("select * from " . DIA_TABLE_COMMENTS . " where comment_ID ='".$r->note_comment_ID."' and comment_content = '".$r->note_text."'");
						$list = $wpdb->get_results( $wpdb->prepare("SELECT * from ".DIA_TABLE_COMMENTS." WHERE comment_ID = %s AND comment_content = %s", $r->note_comment_ID, $r->note_text));
						foreach ($list as $t) {
							$postid = $t->comment_post_ID;
						}
					}
					if($postid == 0){
						//grab post ID from note image ID
						$postid = dia_getBetween('-','-',$r->note_img_ID);
					}
					
					if($postid != 0){
						//find all images
						$post = get_post($postid);
						$postcontent = $post->post_content;
						preg_match_all('/<img[^>]+>/i', $postcontent, $matches);
						//print_r($matches);
						
						//find id and src
						$img = array();
						foreach( $matches[0] as $img_tag){
							preg_match_all('/(id|src)=("[^"]*")/i',$img_tag, $img[$img_tag]);
						}
						//print_r($img);
						
						$imgid;
						$imgsrc;
						$count;
						foreach( $img as $img_tag){
							$first = trim($img_tag[0][0]);
							$second = trim($img_tag[0][1]);
							
							if(strpos($first,'id=')!==false){
								$imgid = substr($first, 4, -1);
								$imgsrc = substr($second, 5, -1);
							}else if(strpos($first,'src=')!==false){
								$imgsrc = substr($first, 5, -1);
								$imgid=md5($imgsrc);
								$imgid='img-'.$postid.'-'.substr($imgid,0,10);
							}
							
							if($r->note_img_ID == $imgid){
								$notelink = sprintf('?page=%s&action=%s&note=%s&paged=%s"',esc_textarea($page),strtolower($condition),esc_textarea($item->note_ID), esc_textarea($paged));
								echo '<div id="dia-admin-holder" data-note-ID="'.esc_attr($r->note_ID).'" date-note-link="'.esc_attr($notelink).'">';
								echo '<img id="'.esc_attr($imgid).'" addable="false" src="'.esc_textarea($imgsrc).'">';	
								echo '</div>';
							}
							$count++;
						}
						if($count==0){
							esc_html_e( 'No preview image' , 'demon-image-annotation');
						}
					}else{
						esc_html_e( 'No preview image' , 'demon-image-annotation');
					}
										
					echo '</td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Author' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_author" type="text" size="40" value="'.esc_textarea($r->note_author).'" /></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Email' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_email" type="text" size="40" value="'.esc_textarea($r->note_email).'" /></td>';
					echo '</tr>';
					
					echo '<tr>';
					echo '<td>'.esc_html__( 'Top' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_top" type="text" size="5" value="'.esc_attr($r->note_top).'" /></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Left' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_left" type="text" size="5" value="'.esc_attr($r->note_left).'" /></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Width' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_width" type="text" size="5" value="'.esc_attr($r->note_width).'" /></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Height' , 'demon-image-annotation').'</td>';
					echo '<td><input name="note_height" type="text" size="5" value="'.esc_attr($r->note_height).'" /></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td>'.esc_html__( 'Text' , 'demon-image-annotation').'</td>';
					echo '<td><textarea name="note_text" cols="32" rows="5">'.esc_textarea($r->note_text).'</textarea></td>';
					echo '</tr>';
					echo '<tr>';
					echo '<td></td>';
					echo '<td><input type="submit" name="update" value="'.esc_html__( 'Update' , 'demon-image-annotation').'" class="button-primary action" /> <a class="button-secondary" href="?page='.esc_textarea($page).'&paged='.esc_textarea($paged).'">'.esc_html__( 'Cancel' , 'demon-image-annotation').'</a></td>';
					echo '</tr>';
				}
				echo '</tbody>';
				echo "</table>";
				
				?></form></div><?php
			}
		}
	}
	
	/**
	 * Prepare the table with different parameters, pagination, columns and table elements
	 */
	function prepare_items() {
		global $wpdb, $_wp_column_headers, $screen, $allnum, $pendingnum, $approvednum;
		$screen = get_current_screen();
		$filter = ( isset($_REQUEST['filter']) ? sanitize_text_field($_REQUEST['filter']) : 'all');
		
		/* Handle our bulk actions */
			$this->process_bulk_action();
			
		/* -- Preparing your query -- */
			if( (get_option('demon_image_annotation_comments') == '0') ) {
				//with wordpress comment
				$queryall = "SELECT ".DIA_TABLE_NOTES.".*, ".DIA_TABLE_COMMENTS.".comment_approved FROM ".DIA_TABLE_NOTES." LEFT OUTER JOIN ".DIA_TABLE_COMMENTS." on ".DIA_TABLE_COMMENTS.".comment_ID = ".DIA_TABLE_NOTES.".note_comment_ID WHERE ".DIA_TABLE_NOTES.".note_comment_ID != 0";
				$querypending = "SELECT ".DIA_TABLE_NOTES.".*, ".DIA_TABLE_COMMENTS.".comment_approved FROM ".DIA_TABLE_NOTES." LEFT OUTER JOIN ".DIA_TABLE_COMMENTS." on ".DIA_TABLE_COMMENTS.".comment_ID = ".DIA_TABLE_NOTES.".note_comment_ID WHERE ".DIA_TABLE_NOTES.".note_comment_ID != 0 AND note_approved = 0";
				$queryapproved = "SELECT ".DIA_TABLE_NOTES.".*, ".DIA_TABLE_COMMENTS.".comment_approved FROM ".DIA_TABLE_NOTES." LEFT OUTER JOIN ".DIA_TABLE_COMMENTS." on ".DIA_TABLE_COMMENTS.".comment_ID = ".DIA_TABLE_NOTES.".note_comment_ID WHERE ".DIA_TABLE_NOTES.".note_comment_ID != 0 AND note_approved = 1";
			} else {
				$queryall = "SELECT * FROM ".DIA_TABLE_NOTES." WHERE ".DIA_TABLE_NOTES.".note_comment_ID = 0";
				$querypending = "SELECT * FROM ".DIA_TABLE_NOTES." WHERE ".DIA_TABLE_NOTES.".note_comment_ID = 0 AND note_approved = 0";
				$queryapproved = "SELECT * FROM ".DIA_TABLE_NOTES." WHERE ".DIA_TABLE_NOTES.".note_comment_ID = 0 AND note_approved = 1";
			}
			
			$wpdb->get_results( $queryall );
			$allnum = $wpdb->num_rows;
			
			$wpdb->get_results( $querypending );
			$pendingnum = $wpdb->num_rows;
			
			$wpdb->get_results( $queryapproved );
			$approvednum = $wpdb->num_rows;
			
			$additionalquery = '';
			if($filter == 'pending'){
				$additionalquery = ' AND note_approved = 0';
			}else if($filter == 'approved'){
				$additionalquery = ' AND note_approved = 1';
			}
			
			//check is sync with wordpress comment
			if( (get_option('demon_image_annotation_comments') == '0') ) {
				//with wordpress comment
				$query = "SELECT ".DIA_TABLE_NOTES.".*, ".DIA_TABLE_COMMENTS.".comment_approved FROM ".DIA_TABLE_NOTES." LEFT OUTER JOIN ".DIA_TABLE_COMMENTS." on ".DIA_TABLE_COMMENTS.".comment_ID = ".DIA_TABLE_NOTES.".note_comment_ID WHERE ".DIA_TABLE_NOTES.".note_comment_ID != 0".$additionalquery;
			} else {
				$query = "SELECT * FROM ".DIA_TABLE_NOTES." WHERE ".DIA_TABLE_NOTES.".note_comment_ID = 0".$additionalquery;
			}
			
		/* -- Ordering parameters -- */
			//Parameters that are going to be used to order the result
			$orderby = !empty($_GET["orderby"]) ? sanitize_text_field($_GET["orderby"]) : 'ASC';
			$order = !empty($_GET["order"]) ? sanitize_text_field($_GET["order"]) : '';
			if(!empty($orderby) & !empty($order)){ $query.=' ORDER BY '.$orderby.' '.$order; } else { $query.= ' ORDER BY note_ID DESC';}
	
		/* -- Pagination parameters -- */
			//Number of elements in your table?
			$totalitems = $wpdb->query($query); //return the total number of affected rows
			//How many to display per page?
			$perpage = 10;
			//Which page is this?
			$paged = !empty($_GET["paged"]) ? sanitize_text_field($_GET["paged"]) : '';
			//Page Number
			if(empty($paged) || !is_numeric($paged) || $paged<=0 ){ $paged=1; }
			//How many pages do we have in total?
			$totalpages = ceil($totalitems/$perpage);
			//adjust the query to take pagination into account
			if(!empty($paged) && !empty($perpage)){
				$offset=($paged-1)*$perpage;
				$query.=' LIMIT '.(int)$offset.','.(int)$perpage;
			}
	
		/* -- Register the pagination -- */
			$this->set_pagination_args( array(
				"total_items" => $totalitems,
				"total_pages" => $totalpages,
				"per_page" => $perpage,
			) );
			//The pagination links are automatically built according to those parameters
	
		/* -- Register the Columns -- */
			$columns = $this->get_columns();
			$hidden = array();
			$sortable = $this->get_sortable_columns();
			$this->_column_headers = array($columns, $hidden, $sortable);
			
		/* -- Fetch the items -- */
			$this->items = $wpdb->get_results($query);
	}

} //class
?>